@@ -29,6 +29,8 @@ import org.apache.parquet.io.ParquetDecodingException;
 import org.apache.parquet.io.api.Binary;
 import org.apache.parquet.io.api.GroupConverter;
 import org.apache.parquet.io.api.PrimitiveConverter;
+import org.apache.parquet.schema.PrimitiveStringifier;
+import org.apache.parquet.schema.PrimitiveType;
 
 public class AvroConverters {
 
@@ -273,13 +275,8 @@ public class AvroConverters {
     public Object convert(Binary binary) {
       try {
         return ctor.newInstance(binary.toStringUsingUTF8());
-      } catch (InstantiationException e) {
-        throw new ParquetDecodingException(
-            "Cannot convert binary to " + stringableName, e);
-      } catch (IllegalAccessException e) {
-        throw new ParquetDecodingException(
-            "Cannot convert binary to " + stringableName, e);
-      } catch (InvocationTargetException e) {
+      } catch (InstantiationException | IllegalAccessException
+          | InvocationTargetException e) {
         throw new ParquetDecodingException(
             "Cannot convert binary to " + stringableName, e);
       }
@@ -319,4 +316,18 @@ public class AvroConverters {
       return model.createFixed(null /* reuse */, binary.getBytes(), schema);
     }
   }
+
+  static final class FieldUUIDConverter extends BinaryConverter<String> {
+    private final PrimitiveStringifier stringifier;
+
+    public FieldUUIDConverter(ParentValueContainer parent, PrimitiveType type) {
+      super(parent);
+      stringifier = type.stringifier();
+    }
+
+    @Override
+    public String convert(Binary binary) {
+      return stringifier.stringify(binary);
+    }
+  }
 }
