@@ -21,6 +21,7 @@ package org.apache.parquet.schema;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
+import java.util.Objects;
 import java.util.Optional;
 
 import org.apache.parquet.Preconditions;
@@ -237,8 +238,7 @@ public class Types {
     protected final THIS repetition(Type.Repetition repetition) {
       Preconditions.checkArgument(!repetitionAlreadySet,
           "Repetition has already been set");
-      Preconditions.checkNotNull(repetition, "Repetition cannot be null");
-      this.repetition = repetition;
+      this.repetition = Objects.requireNonNull(repetition, "Repetition cannot be null");
       this.repetitionAlreadySet = true;
       return self();
     }
@@ -310,8 +310,8 @@ public class Types {
      * @return the parent {@code GroupBuilder} or the constructed {@code Type}
      */
     public P named(String name) {
-      Preconditions.checkNotNull(name, "Name is required");
-      Preconditions.checkNotNull(repetition, "Repetition is required");
+      Objects.requireNonNull(name, "Name is required");
+      Objects.requireNonNull(repetition, "Repetition is required");
 
       Type type = build(name);
       if (parent != null) {
@@ -447,20 +447,22 @@ public class Types {
         logicalTypeAnnotation.accept(new LogicalTypeAnnotation.LogicalTypeAnnotationVisitor<Boolean>() {
           @Override
           public Optional<Boolean> visit(LogicalTypeAnnotation.StringLogicalTypeAnnotation stringLogicalType) {
-            checkBinaryPrimitiveType(stringLogicalType);
-            return Optional.of(true);
+            return checkBinaryPrimitiveType(stringLogicalType);
           }
 
           @Override
           public Optional<Boolean> visit(LogicalTypeAnnotation.JsonLogicalTypeAnnotation jsonLogicalType) {
-            checkBinaryPrimitiveType(jsonLogicalType);
-            return Optional.of(true);
+            return checkBinaryPrimitiveType(jsonLogicalType);
           }
 
           @Override
           public Optional<Boolean> visit(LogicalTypeAnnotation.BsonLogicalTypeAnnotation bsonLogicalType) {
-            checkBinaryPrimitiveType(bsonLogicalType);
-            return Optional.of(true);
+            return checkBinaryPrimitiveType(bsonLogicalType);
+          }
+
+          @Override
+          public Optional<Boolean> visit(LogicalTypeAnnotation.UUIDLogicalTypeAnnotation uuidLogicalType) {
+            return checkFixedPrimitiveType(LogicalTypeAnnotation.UUIDLogicalTypeAnnotation.BYTES, uuidLogicalType);
           }
 
           @Override
@@ -497,8 +499,7 @@ public class Types {
 
           @Override
           public Optional<Boolean> visit(LogicalTypeAnnotation.DateLogicalTypeAnnotation dateLogicalType) {
-            checkInt32PrimitiveType(dateLogicalType);
-            return Optional.of(true);
+            return checkInt32PrimitiveType(dateLogicalType);
           }
 
           @Override
@@ -538,41 +539,43 @@ public class Types {
 
           @Override
           public Optional<Boolean> visit(LogicalTypeAnnotation.TimestampLogicalTypeAnnotation timestampLogicalType) {
-            checkInt64PrimitiveType(timestampLogicalType);
-            return Optional.of(true);
+            return checkInt64PrimitiveType(timestampLogicalType);
           }
 
           @Override
           public Optional<Boolean> visit(LogicalTypeAnnotation.IntervalLogicalTypeAnnotation intervalLogicalType) {
-            Preconditions.checkState(
-                (primitiveType == PrimitiveTypeName.FIXED_LEN_BYTE_ARRAY) &&
-                (length == 12),
-                "INTERVAL can only annotate FIXED_LEN_BYTE_ARRAY(12)");
-            return Optional.of(true);
+            return checkFixedPrimitiveType(12, intervalLogicalType);
           }
 
           @Override
           public Optional<Boolean> visit(LogicalTypeAnnotation.EnumLogicalTypeAnnotation enumLogicalType) {
+            return checkBinaryPrimitiveType(enumLogicalType);
+          }
+
+          private Optional<Boolean> checkFixedPrimitiveType(int l, LogicalTypeAnnotation logicalTypeAnnotation) {
             Preconditions.checkState(
-                primitiveType == PrimitiveTypeName.BINARY,
-                "ENUM can only annotate binary fields");
+                primitiveType == PrimitiveTypeName.FIXED_LEN_BYTE_ARRAY && length == l,
+              logicalTypeAnnotation.toString() + " can only annotate FIXED_LEN_BYTE_ARRAY(" + l + ')');
             return Optional.of(true);
           }
 
-          private void checkBinaryPrimitiveType(LogicalTypeAnnotation logicalTypeAnnotation) {
+          private Optional<Boolean> checkBinaryPrimitiveType(LogicalTypeAnnotation logicalTypeAnnotation) {
             Preconditions.checkState(
                 primitiveType == PrimitiveTypeName.BINARY,
-              logicalTypeAnnotation.toString() + " can only annotate binary fields");
+              logicalTypeAnnotation.toString() + " can only annotate BINARY");
+            return Optional.of(true);
           }
 
-          private void checkInt32PrimitiveType(LogicalTypeAnnotation logicalTypeAnnotation) {
+          private Optional<Boolean> checkInt32PrimitiveType(LogicalTypeAnnotation logicalTypeAnnotation) {
             Preconditions.checkState(primitiveType == PrimitiveTypeName.INT32,
               logicalTypeAnnotation.toString() + " can only annotate INT32");
+            return Optional.of(true);
           }
 
-          private void checkInt64PrimitiveType(LogicalTypeAnnotation logicalTypeAnnotation) {
+          private Optional<Boolean> checkInt64PrimitiveType(LogicalTypeAnnotation logicalTypeAnnotation) {
             Preconditions.checkState(primitiveType == PrimitiveTypeName.INT64,
               logicalTypeAnnotation.toString() + " can only annotate INT64");
+            return Optional.of(true);
           }
         }).orElseThrow(() -> new IllegalStateException(logicalTypeAnnotation + " can not be applied to a primitive type"));
       }
@@ -648,12 +651,12 @@ public class Types {
 
     private BaseGroupBuilder(P parent) {
       super(parent);
-      this.fields = new ArrayList<Type>();
+      this.fields = new ArrayList<>();
     }
 
     private BaseGroupBuilder(Class<P> returnType) {
       super(returnType);
-      this.fields = new ArrayList<Type>();
+      this.fields = new ArrayList<>();
     }
 
     @Override
@@ -661,7 +664,7 @@ public class Types {
 
     public PrimitiveBuilder<THIS> primitive(
         PrimitiveTypeName type, Type.Repetition repetition) {
-      return new PrimitiveBuilder<THIS> (self(), type)
+      return new PrimitiveBuilder<> (self(), type)
           .repetition(repetition);
     }
 
@@ -675,7 +678,7 @@ public class Types {
      */
     public PrimitiveBuilder<THIS> required(
         PrimitiveTypeName type) {
-      return new PrimitiveBuilder<THIS>(self(), type)
+      return new PrimitiveBuilder<>(self(), type)
           .repetition(Type.Repetition.REQUIRED);
     }
 
@@ -689,7 +692,7 @@ public class Types {
      */
     public PrimitiveBuilder<THIS> optional(
         PrimitiveTypeName type) {
-      return new PrimitiveBuilder<THIS>(self(), type)
+      return new PrimitiveBuilder<>(self(), type)
           .repetition(Type.Repetition.OPTIONAL);
     }
 
@@ -703,12 +706,12 @@ public class Types {
      */
     public PrimitiveBuilder<THIS> repeated(
         PrimitiveTypeName type) {
-      return new PrimitiveBuilder<THIS>(self(), type)
+      return new PrimitiveBuilder<>(self(), type)
           .repetition(Type.Repetition.REPEATED);
     }
 
     public GroupBuilder<THIS> group(Type.Repetition repetition) {
-      return new GroupBuilder<THIS>(self())
+      return new GroupBuilder<>(self())
           .repetition(repetition);
     }
 
@@ -719,7 +722,7 @@ public class Types {
      *          fields.
      */
     public GroupBuilder<THIS> requiredGroup() {
-      return new GroupBuilder<THIS>(self())
+      return new GroupBuilder<>(self())
           .repetition(Type.Repetition.REQUIRED);
     }
 
@@ -730,7 +733,7 @@ public class Types {
      *          fields.
      */
     public GroupBuilder<THIS> optionalGroup() {
-      return new GroupBuilder<THIS>(self())
+      return new GroupBuilder<>(self())
           .repetition(Type.Repetition.OPTIONAL);
     }
 
@@ -741,7 +744,7 @@ public class Types {
      *          fields.
      */
     public GroupBuilder<THIS> repeatedGroup() {
-      return new GroupBuilder<THIS>(self())
+      return new GroupBuilder<>(self())
           .repetition(Type.Repetition.REPEATED);
     }
 
@@ -778,21 +781,21 @@ public class Types {
 
     public MapBuilder<THIS> map(
         Type.Repetition repetition) {
-      return new MapBuilder<THIS>(self()).repetition(repetition);
+      return new MapBuilder<>(self()).repetition(repetition);
     }
 
     public MapBuilder<THIS> requiredMap() {
-      return new MapBuilder<THIS>(self())
+      return new MapBuilder<>(self())
           .repetition(Type.Repetition.REQUIRED);
     }
 
     public MapBuilder<THIS> optionalMap() {
-      return new MapBuilder<THIS>(self())
+      return new MapBuilder<>(self())
           .repetition(Type.Repetition.OPTIONAL);
     }
 
     public ListBuilder<THIS> list(Type.Repetition repetition) {
-      return new ListBuilder<THIS>(self()).repetition(repetition);
+      return new ListBuilder<>(self()).repetition(repetition);
     }
 
     public ListBuilder<THIS> requiredList() {
@@ -844,7 +847,7 @@ public class Types {
       public ValueBuilder<MP, M> value(PrimitiveTypeName type,
                                        Type.Repetition repetition) {
         mapBuilder.setKeyType(build("key"));
-        return new ValueBuilder<MP, M>(mapBuilder, type).repetition(repetition);
+        return new ValueBuilder<>(mapBuilder, type).repetition(repetition);
       }
 
       public ValueBuilder<MP, M> requiredValue(PrimitiveTypeName type) {
@@ -857,7 +860,7 @@ public class Types {
 
       public GroupValueBuilder<MP, M> groupValue(Type.Repetition repetition) {
         mapBuilder.setKeyType(build("key"));
-        return new GroupValueBuilder<MP, M>(mapBuilder).repetition(repetition);
+        return new GroupValueBuilder<>(mapBuilder).repetition(repetition);
       }
 
       public GroupValueBuilder<MP, M> requiredGroupValue() {
@@ -870,7 +873,7 @@ public class Types {
 
       public MapValueBuilder<MP, M> mapValue(Type.Repetition repetition) {
         mapBuilder.setKeyType(build("key"));
-        return new MapValueBuilder<MP, M>(mapBuilder).repetition(repetition);
+        return new MapValueBuilder<>(mapBuilder).repetition(repetition);
       }
 
       public MapValueBuilder<MP, M> requiredMapValue() {
@@ -883,7 +886,7 @@ public class Types {
 
       public ListValueBuilder<MP, M> listValue(Type.Repetition repetition) {
         mapBuilder.setKeyType(build("key"));
-        return new ListValueBuilder<MP, M>(mapBuilder).repetition(repetition);
+        return new ListValueBuilder<>(mapBuilder).repetition(repetition);
       }
 
       public ListValueBuilder<MP, M> requiredListValue() {
@@ -921,6 +924,7 @@ public class Types {
         this.mapBuilder = mapBuilder;
       }
 
+      @Override
       public MP named(String name) {
         mapBuilder.setValueType(build("value"));
         return mapBuilder.named(name);
@@ -950,7 +954,7 @@ public class Types {
       public ValueBuilder<MP, M> value(PrimitiveTypeName type,
                                        Type.Repetition repetition) {
         mapBuilder.setKeyType(build("key"));
-        return new ValueBuilder<MP, M>(mapBuilder, type).repetition(repetition);
+        return new ValueBuilder<>(mapBuilder, type).repetition(repetition);
       }
 
       public ValueBuilder<MP, M> requiredValue(PrimitiveTypeName type) {
@@ -963,7 +967,7 @@ public class Types {
 
       public GroupValueBuilder<MP, M> groupValue(Type.Repetition repetition) {
         mapBuilder.setKeyType(build("key"));
-        return new GroupValueBuilder<MP, M>(mapBuilder).repetition(repetition);
+        return new GroupValueBuilder<>(mapBuilder).repetition(repetition);
       }
 
       public GroupValueBuilder<MP, M> requiredGroupValue() {
@@ -976,7 +980,7 @@ public class Types {
 
       public MapValueBuilder<MP, M> mapValue(Type.Repetition repetition) {
         mapBuilder.setKeyType(build("key"));
-        return new MapValueBuilder<MP, M>(mapBuilder).repetition(repetition);
+        return new MapValueBuilder<>(mapBuilder).repetition(repetition);
       }
 
       public MapValueBuilder<MP, M> requiredMapValue() {
@@ -989,7 +993,7 @@ public class Types {
 
       public ListValueBuilder<MP, M> listValue(Type.Repetition repetition) {
         mapBuilder.setKeyType(build("key"));
-        return new ListValueBuilder<MP, M>(mapBuilder).repetition(repetition);
+        return new ListValueBuilder<>(mapBuilder).repetition(repetition);
       }
 
       public ListValueBuilder<MP, M> requiredListValue() {
@@ -1022,6 +1026,7 @@ public class Types {
         this.mapBuilder = mapBuilder;
       }
 
+      @Override
       public MP named(String name) {
         mapBuilder.setValueType(build("value"));
         return mapBuilder.named(name);
@@ -1042,6 +1047,7 @@ public class Types {
         this.mapBuilder = mapBuilder;
       }
 
+      @Override
       public MP named(String name) {
         mapBuilder.setValueType(build("value"));
         return mapBuilder.named(name);
@@ -1062,6 +1068,7 @@ public class Types {
         this.mapBuilder = mapBuilder;
       }
 
+      @Override
       public MP named(String name) {
         mapBuilder.setValueType(build("value"));
         return mapBuilder.named(name);
@@ -1100,7 +1107,7 @@ public class Types {
     protected abstract THIS self();
 
     public KeyBuilder<P, THIS> key(PrimitiveTypeName type) {
-      return new KeyBuilder<P, THIS>(self(), type);
+      return new KeyBuilder<>(self(), type);
     }
 
     public THIS key(Type type) {
@@ -1109,12 +1116,12 @@ public class Types {
     }
 
     public GroupKeyBuilder<P, THIS> groupKey() {
-      return new GroupKeyBuilder<P, THIS>(self());
+      return new GroupKeyBuilder<>(self());
     }
 
     public ValueBuilder<P, THIS> value(PrimitiveTypeName type,
                                        Type.Repetition repetition) {
-      return new ValueBuilder<P, THIS>(self(), type).repetition(repetition);
+      return new ValueBuilder<>(self(), type).repetition(repetition);
     }
 
     public ValueBuilder<P, THIS> requiredValue(PrimitiveTypeName type) {
@@ -1126,7 +1133,7 @@ public class Types {
     }
 
     public GroupValueBuilder<P, THIS> groupValue(Type.Repetition repetition) {
-      return new GroupValueBuilder<P, THIS>(self()).repetition(repetition);
+      return new GroupValueBuilder<>(self()).repetition(repetition);
     }
 
     public GroupValueBuilder<P, THIS> requiredGroupValue() {
@@ -1138,7 +1145,7 @@ public class Types {
     }
 
     public MapValueBuilder<P, THIS> mapValue(Type.Repetition repetition) {
-      return new MapValueBuilder<P, THIS>(self()).repetition(repetition);
+      return new MapValueBuilder<>(self()).repetition(repetition);
     }
 
     public MapValueBuilder<P, THIS> requiredMapValue() {
@@ -1150,7 +1157,7 @@ public class Types {
     }
 
     public ListValueBuilder<P, THIS> listValue(Type.Repetition repetition) {
-      return new ListValueBuilder<P, THIS>(self()).repetition(repetition);
+      return new ListValueBuilder<>(self()).repetition(repetition);
     }
 
     public ListValueBuilder<P, THIS> requiredListValue() {
@@ -1236,6 +1243,7 @@ public class Types {
         this.listBuilder = listBuilder;
       }
 
+      @Override
       public LP named(String name) {
         listBuilder.setElementType(build("element"));
         return listBuilder.named(name);
@@ -1256,6 +1264,7 @@ public class Types {
         this.listBuilder = listBuilder;
       }
 
+      @Override
       public LP named(String name) {
         listBuilder.setElementType(build("element"));
         return listBuilder.named(name);
@@ -1318,7 +1327,7 @@ public class Types {
     protected Type build(String name) {
       Preconditions.checkState(logicalTypeAnnotation == null,
           "LIST is already the logical type and can't be changed");
-      Preconditions.checkNotNull(elementType, "List element type");
+      Objects.requireNonNull(elementType, "List element type cannot be null");
 
       GroupBuilder<GroupType> builder = buildGroup(repetition).as(OriginalType.LIST);
       if (id != null) {
@@ -1332,7 +1341,7 @@ public class Types {
 
     public ElementBuilder<P, THIS> element(PrimitiveTypeName type,
                                            Type.Repetition repetition) {
-      return new ElementBuilder<P, THIS>(self(), type).repetition(repetition);
+      return new ElementBuilder<>(self(), type).repetition(repetition);
     }
 
     public ElementBuilder<P, THIS> requiredElement(PrimitiveTypeName type) {
@@ -1344,7 +1353,7 @@ public class Types {
     }
 
     public GroupElementBuilder<P, THIS> groupElement(Type.Repetition repetition) {
-      return new GroupElementBuilder<P, THIS>(self()).repetition(repetition);
+      return new GroupElementBuilder<>(self()).repetition(repetition);
     }
 
     public GroupElementBuilder<P, THIS> requiredGroupElement() {
@@ -1356,7 +1365,7 @@ public class Types {
     }
 
     public MapElementBuilder<P, THIS> mapElement(Type.Repetition repetition) {
-      return new MapElementBuilder<P, THIS>(self()).repetition(repetition);
+      return new MapElementBuilder<>(self()).repetition(repetition);
     }
 
     public MapElementBuilder<P, THIS> requiredMapElement() {
@@ -1368,7 +1377,7 @@ public class Types {
     }
 
     public ListElementBuilder<P, THIS> listElement(Type.Repetition repetition) {
-      return new ListElementBuilder<P, THIS>(self()).repetition(repetition);
+      return new ListElementBuilder<>(self()).repetition(repetition);
     }
 
     public ListElementBuilder<P, THIS> requiredListElement() {
@@ -1417,7 +1426,7 @@ public class Types {
      */
     @Override
     public MessageType named(String name) {
-      Preconditions.checkNotNull(name, "Name is required");
+      Objects.requireNonNull(name, "Name is required");
       return new MessageType(name, fields);
     }
   }
@@ -1433,7 +1442,7 @@ public class Types {
 
   public static PrimitiveBuilder<PrimitiveType> primitive(PrimitiveTypeName type,
                                                           Type.Repetition repetition) {
-    return new PrimitiveBuilder<PrimitiveType>(PrimitiveType.class, type)
+    return new PrimitiveBuilder<>(PrimitiveType.class, type)
         .repetition(repetition);
   }
 
@@ -1444,7 +1453,7 @@ public class Types {
    * @return a {@link PrimitiveBuilder}
    */
   public static PrimitiveBuilder<PrimitiveType> required(PrimitiveTypeName type) {
-    return new PrimitiveBuilder<PrimitiveType>(PrimitiveType.class, type)
+    return new PrimitiveBuilder<>(PrimitiveType.class, type)
         .repetition(Type.Repetition.REQUIRED);
   }
 
@@ -1455,7 +1464,7 @@ public class Types {
    * @return a {@link PrimitiveBuilder}
    */
   public static PrimitiveBuilder<PrimitiveType> optional(PrimitiveTypeName type) {
-    return new PrimitiveBuilder<PrimitiveType>(PrimitiveType.class, type)
+    return new PrimitiveBuilder<>(PrimitiveType.class, type)
         .repetition(Type.Repetition.OPTIONAL);
   }
 
@@ -1466,13 +1475,13 @@ public class Types {
    * @return a {@link PrimitiveBuilder}
    */
   public static PrimitiveBuilder<PrimitiveType> repeated(PrimitiveTypeName type) {
-    return new PrimitiveBuilder<PrimitiveType>(PrimitiveType.class, type)
+    return new PrimitiveBuilder<>(PrimitiveType.class, type)
         .repetition(Type.Repetition.REPEATED);
   }
 
   public static GroupBuilder<GroupType> buildGroup(
       Type.Repetition repetition) {
-    return new GroupBuilder<GroupType>(GroupType.class).repetition(repetition);
+    return new GroupBuilder<>(GroupType.class).repetition(repetition);
   }
 
   /**
@@ -1481,7 +1490,7 @@ public class Types {
    * @return a {@link GroupBuilder}
    */
   public static GroupBuilder<GroupType> requiredGroup() {
-    return new GroupBuilder<GroupType>(GroupType.class)
+    return new GroupBuilder<>(GroupType.class)
         .repetition(Type.Repetition.REQUIRED);
   }
 
@@ -1491,7 +1500,7 @@ public class Types {
    * @return a {@link GroupBuilder}
    */
   public static GroupBuilder<GroupType> optionalGroup() {
-    return new GroupBuilder<GroupType>(GroupType.class)
+    return new GroupBuilder<>(GroupType.class)
         .repetition(Type.Repetition.OPTIONAL);
   }
 
@@ -1501,13 +1510,13 @@ public class Types {
    * @return a {@link GroupBuilder}
    */
   public static GroupBuilder<GroupType> repeatedGroup() {
-    return new GroupBuilder<GroupType>(GroupType.class)
+    return new GroupBuilder<>(GroupType.class)
         .repetition(Type.Repetition.REPEATED);
   }
 
 
   public static MapBuilder<GroupType> map(Type.Repetition repetition) {
-    return new MapBuilder<GroupType>(GroupType.class).repetition(repetition);
+    return new MapBuilder<>(GroupType.class).repetition(repetition);
   }
 
   public static MapBuilder<GroupType> requiredMap() {
@@ -1519,7 +1528,7 @@ public class Types {
   }
 
   public static ListBuilder<GroupType> list(Type.Repetition repetition) {
-    return new ListBuilder<GroupType>(GroupType.class).repetition(repetition);
+    return new ListBuilder<>(GroupType.class).repetition(repetition);
   }
 
   public static ListBuilder<GroupType> requiredList() {
