@@ -303,7 +303,7 @@ public class ParquetFileWriter {
       int statisticsTruncateLength, boolean pageWriteChecksumEnabled)
           throws IOException{
     this(file, schema, mode, rowGroupSize, maxPaddingSize, columnIndexTruncateLength,
-      statisticsTruncateLength, pageWriteChecksumEnabled, null);
+      statisticsTruncateLength, pageWriteChecksumEnabled, null, null);
   }
 
   public ParquetFileWriter(OutputFile file, MessageType schema, Mode mode,
@@ -311,6 +311,24 @@ public class ParquetFileWriter {
                            int statisticsTruncateLength, boolean pageWriteChecksumEnabled,
                            FileEncryptionProperties encryptionProperties)
     throws IOException {
+    this(file, schema, mode, rowGroupSize, maxPaddingSize, columnIndexTruncateLength,
+      statisticsTruncateLength, pageWriteChecksumEnabled, encryptionProperties, null);
+  }
+
+  public ParquetFileWriter(OutputFile file, MessageType schema, Mode mode,
+                           long rowGroupSize, int maxPaddingSize, int columnIndexTruncateLength,
+                           int statisticsTruncateLength, boolean pageWriteChecksumEnabled,
+                           InternalFileEncryptor encryptor)
+    throws IOException {
+    this(file, schema, mode, rowGroupSize, maxPaddingSize, columnIndexTruncateLength,
+      statisticsTruncateLength, pageWriteChecksumEnabled, null, encryptor);
+  }
+
+  private ParquetFileWriter(OutputFile file, MessageType schema, Mode mode,
+                           long rowGroupSize, int maxPaddingSize, int columnIndexTruncateLength,
+                           int statisticsTruncateLength, boolean pageWriteChecksumEnabled,
+                           FileEncryptionProperties encryptionProperties, InternalFileEncryptor encryptor)
+    throws IOException {
     TypeUtil.checkValidWriteSchema(schema);
 
     this.schema = schema;
@@ -336,20 +354,30 @@ public class ParquetFileWriter {
 
     this.metadataConverter = new ParquetMetadataConverter(statisticsTruncateLength);
 
-    if (null == encryptionProperties) {
+    if (null == encryptionProperties && null == encryptor) {
       this.fileEncryptor = null;
-    } else {
-      // Verify that every encrypted column is in file schema
-      Map<ColumnPath, ColumnEncryptionProperties> columnEncryptionProperties = encryptionProperties.getEncryptedColumns();
-      if (null != columnEncryptionProperties) { // if null, every column in file schema will be encrypted with footer key
-        for (Map.Entry<ColumnPath, ColumnEncryptionProperties> entry : columnEncryptionProperties.entrySet()) {
-          String[] path = entry.getKey().toArray();
-          if(!schema.containsPath(path)) {
-            throw new ParquetCryptoRuntimeException("Encrypted column " + Arrays.toString(path) + " not in file schema");
-          }
+      return;
+    }
+
+    if (null == encryptionProperties) {
+      encryptionProperties = encryptor.getEncryptionProperties();
+    }
+
+    // Verify that every encrypted column is in file schema
+    Map<ColumnPath, ColumnEncryptionProperties> columnEncryptionProperties = encryptionProperties.getEncryptedColumns();
+    if (null != columnEncryptionProperties) { // if null, every column in file schema will be encrypted with footer key
+      for (Map.Entry<ColumnPath, ColumnEncryptionProperties> entry : columnEncryptionProperties.entrySet()) {
+        String[] path = entry.getKey().toArray();
+        if (!schema.containsPath(path)) {
+          throw new ParquetCryptoRuntimeException("Encrypted column " + Arrays.toString(path) + " not in file schema");
         }
       }
+    }
+
+    if (null == encryptor) {
       this.fileEncryptor = new InternalFileEncryptor(encryptionProperties);
+    } else {
+      this.fileEncryptor = encryptor;
     }
   }
 
@@ -394,7 +422,7 @@ public class ParquetFileWriter {
     out.write(magic);
   }
 
-  InternalFileEncryptor getEncryptor() {
+  public InternalFileEncryptor getEncryptor() {
     return fileEncryptor;
   }
 
@@ -563,7 +591,7 @@ public class ParquetFileWriter {
     // We are unable to build indexes without rowCount so skip them for this column
     offsetIndexBuilder = OffsetIndexBuilder.getNoOpBuilder();
     columnIndexBuilder = ColumnIndexBuilder.getNoOpBuilder();
-    innerWriteDataPage(valueCount, uncompressedPageSize, bytes, statistics, rlEncoding, dlEncoding, valuesEncoding);
+    innerWriteDataPage(valueCount, uncompressedPageSize, bytes, statistics, rlEncoding, dlEncoding, valuesEncoding, null, null);
   }
 
   /**
@@ -578,6 +606,31 @@ public class ParquetFileWriter {
    * @param valuesEncoding encoding of values
    * @throws IOException if any I/O error occurs during writing the file
    */
+  public void writeDataPage(
+    int valueCount, int uncompressedPageSize,
+    BytesInput bytes,
+    Statistics statistics,
+    long rowCount,
+    Encoding rlEncoding,
+    Encoding dlEncoding,
+    Encoding valuesEncoding) throws IOException {
+    writeDataPage(valueCount, uncompressedPageSize, bytes, statistics, rowCount, rlEncoding, dlEncoding, valuesEncoding, null, null);
+  }
+
+  /**
+   * Writes a single page
+   * @param valueCount count of values
+   * @param uncompressedPageSize the size of the data once uncompressed
+   * @param bytes the compressed data for the page without header
+   * @param statistics the statistics of the page
+   * @param rowCount the number of rows in the page
+   * @param rlEncoding encoding of the repetition level
+   * @param dlEncoding encoding of the definition level
+   * @param valuesEncoding encoding of values
+   * @param metadataBlockEncryptor encryptor for block data
+   * @param pageHeaderAAD pageHeader AAD
+   * @throws IOException if any I/O error occurs during writing the file
+   */
   public void writeDataPage(
       int valueCount, int uncompressedPageSize,
       BytesInput bytes,
@@ -585,10 +638,11 @@ public class ParquetFileWriter {
       long rowCount,
       Encoding rlEncoding,
       Encoding dlEncoding,
-      Encoding valuesEncoding) throws IOException {
+      Encoding valuesEncoding,
+      BlockCipher.Encryptor metadataBlockEncryptor,
+      byte[] pageHeaderAAD) throws IOException {
     long beforeHeader = out.getPos();
-    innerWriteDataPage(valueCount, uncompressedPageSize, bytes, statistics, rlEncoding, dlEncoding, valuesEncoding);
-
+    innerWriteDataPage(valueCount, uncompressedPageSize, bytes, statistics, rlEncoding, dlEncoding, valuesEncoding, metadataBlockEncryptor, pageHeaderAAD);
     offsetIndexBuilder.add((int) (out.getPos() - beforeHeader), rowCount);
   }
 
@@ -598,7 +652,34 @@ public class ParquetFileWriter {
       Statistics statistics,
       Encoding rlEncoding,
       Encoding dlEncoding,
-      Encoding valuesEncoding) throws IOException {
+      Encoding valuesEncoding,
+      BlockCipher.Encryptor metadataBlockEncryptor,
+      byte[] pageHeaderAAD) throws IOException {
+    writeDataPage(valueCount, uncompressedPageSize, bytes, statistics, rlEncoding, dlEncoding, valuesEncoding, metadataBlockEncryptor, pageHeaderAAD);
+  }
+
+  /**
+   * writes a single page
+   * @param valueCount count of values
+   * @param uncompressedPageSize the size of the data once uncompressed
+   * @param bytes the compressed data for the page without header
+   * @param statistics statistics for the page
+   * @param rlEncoding encoding of the repetition level
+   * @param dlEncoding encoding of the definition level
+   * @param valuesEncoding encoding of values
+   * @param metadataBlockEncryptor encryptor for block data
+   * @param pageHeaderAAD pageHeader AAD
+   * @throws IOException if there is an error while writing
+   */
+  public void writeDataPage(
+    int valueCount, int uncompressedPageSize,
+    BytesInput bytes,
+    Statistics statistics,
+    Encoding rlEncoding,
+    Encoding dlEncoding,
+    Encoding valuesEncoding,
+    BlockCipher.Encryptor metadataBlockEncryptor,
+    byte[] pageHeaderAAD) throws IOException {
     state = state.write();
     long beforeHeader = out.getPos();
     if (currentChunkFirstDataPage < 0) {
@@ -616,7 +697,9 @@ public class ParquetFileWriter {
         dlEncoding,
         valuesEncoding,
         (int) crc.getValue(),
-        out);
+        out,
+        metadataBlockEncryptor,
+        pageHeaderAAD);
     } else {
       metadataConverter.writeDataPageV1Header(
         uncompressedPageSize, compressedPageSize,
@@ -624,7 +707,9 @@ public class ParquetFileWriter {
         rlEncoding,
         dlEncoding,
         valuesEncoding,
-        out);
+        out,
+        metadataBlockEncryptor,
+        pageHeaderAAD);
     }
     long headerSize = out.getPos() - beforeHeader;
     this.uncompressedLength += uncompressedPageSize + headerSize;
@@ -1035,6 +1120,12 @@ public class ParquetFileWriter {
     long length = chunk.getTotalSize();
     long newChunkStart = out.getPos();
 
+    if (newChunkStart != start) {
+      offsetIndex = OffsetIndexBuilder.getBuilder()
+        .fromOffsetIndex(offsetIndex)
+        .build(newChunkStart - start);
+    }
+
     copy(from, out, start, length);
 
     currentBloomFilters.put(String.join(".", descriptor.getPath()), bloomFilter);
